import re
import json
import gc
import torch


class Pipeline:
    def __init__(self, dataset, model_h, model_v):
        self.dataset = dataset
        self.model_h = model_h
        self.model_v = model_v
        self.dataset_h = None
        self.dataset_v = None

    def apply(self):
        print("[Pipeline] Starting pipeline...")

        out_h = self.homogenize()

        # prompt_v = "please verify the following text is not empty:"
        # out_v_col = f"{self.dataset.field}__v"
        # dataset_v = self.model_v.run(dataset_h, prompt_v, output_col=out_v_col)
        # print("[Pipeline] Verification completed.")
        # return dataset_v

        print("[Pipeline] Pipeline completed.")
        return out_h

    def homogenize(self) -> str:
        print("[Pipeline - Homogenize] Start Homogenization SS1...")
        out_h_ss1 = self.substep1()
        print("[Pipeline - Homogenize] Homogenization SS1 Done.")

        print("[Pipeline - Homogenize] Start Homogenization SS2...")
        out_h_ss2 = self.substep2(out_h_ss1)
        print("[Pipeline - Homogenize] Homogenization SS2 Done.")
        return out_h_ss2

    def substep1(self) -> dict:
        prompt_h_ss1 = self.build_substep1_prompt()
        max_retries = 3

        for attempt in range(1, max_retries + 1):

            if torch.cuda.is_available():
                torch.cuda.empty_cache()
                gc.collect()

            print(f"[Pipeline - SubStep1] Start Homogenization {attempt}...")

            out_h_col = f"{self.dataset.field}__h"

            self.dataset_h = self.model_h.run(
                self.dataset, prompt_h_ss1, output_col=out_h_col
            )

            out_h_ss1 = self.dataset_h.data.loc[0, out_h_col]

            match = re.search(r"\{.*\}", out_h_ss1, re.DOTALL)
            if match:
                try:
                    parsed = json.loads(match.group(0))
                    if (
                        isinstance(parsed.get("Symptoms"), list)
                        and isinstance(parsed.get("Diagnoses"), list)
                        and isinstance(parsed.get("Treatments"), list)
                    ):
                        return parsed
                except json.JSONDecodeError:
                    pass
            print(f"Retry {attempt}/{max_retries} for JSON extraction...")

            print(f"[Pipeline - SubStep1] Homogenization {attempt} completed.")
        return {"Symptoms": [], "Diagnoses": [], "Treatments": []}

    def substep2(self, entities: dict) -> str:
        prompt_h_ss2 = self.build_substep2_prompt(entities)
        out_h_ss2_col = f"{self.dataset.field}__v"
        self.dataset_h = self.model_h.run(
            self.dataset_h, prompt_h_ss2, output_col=out_h_ss2_col
        )
        out_h_ss2 = self.dataset_h.data.loc[0, out_h_ss2_col]
        return out_h_ss2

    def build_substep1_prompt(self) -> str:
        return (
            "System: You are OpenBioLLM, a clinical text analysis assistant.\n"
            "Output ONLY a JSON object with exactly these keys:\n"
            '- "Symptoms": list of strings (empty if none)\n'
            '- "Diagnoses": list of strings (empty if none)\n'
            '- "Treatments": list of strings (empty if none)\n'
            "Do not add any other keys or text.\n\n"
            "User: Extract the clinical entities from the note below:\n"
        )

    def build_substep2_prompt(self, entities: dict) -> str:
        ent_json = json.dumps(
            {
                "Symptoms": entities["Symptoms"],
                "Diagnoses": entities["Diagnoses"],
                "Treatments": entities["Treatments"],
            },
            ensure_ascii=False,
        )
        return (
            "System: You are OpenBioLLM, a clinical summarization assistant.\n"
            "You will be given a clinical note and a JSON of extracted entities.\n"
            "Write a concise summary of the note, weaving in the key symptoms, diagnoses, and treatments.\n"
            "Output ONLY the summary text, no other JSON or commentary.\n\n"
            f"Extracted Entities:\n{ent_json}\n\n"
            "Clinical Note:"
        )



from __future__ import annotations

from typing import Any, Dict, List, Tuple
import copy
import inspect
import pandas as pd

from transformers import (
    AutoTokenizer,
    AutoModelForCausalLM,
    AutoModelForSeq2SeqLM,
    GenerationConfig,
    pipeline,
)


class Model:
    """
    Model HF générique (DataFrame-only) avec routage des kwargs :
      - kwargs "génération" (ex: max_new_tokens, temperature, top_p, …) → passés au pipeline à l'appel
      - kwargs "model_*"     (ex: model_quantization_config=..., model_device_map="auto", model_torch_dtype=...)
      - kwargs "tokenizer_*" (ex: tokenizer_use_fast=True, tokenizer_padding_side="left")
      - kwargs "pipeline_*"  (ex: pipeline_device_map="auto", pipeline_return_full_text=False, batch_size=...)

    Exemple d'usage plus bas avec Llama3-OpenBioLLM 8B en 8-bit.
    """

    def __init__(self, name: str, task: str = "text-generation", **kwargs: Any):
        self.name = name
        self.task = task

        (
            self.pipeline_kwargs,
            self.generation_kwargs,
            self.model_kwargs,
            self.tokenizer_kwargs,
        ) = self._split_kwargs(kwargs)

        # Politesse: si tu donnes max_new_tokens + max_length, on supprime max_length
        if (
            "max_new_tokens" in self.generation_kwargs
            and "max_length" in self.generation_kwargs
        ):
            self.generation_kwargs.pop("max_length", None)

        self._tokenizer = None
        self._model = None
        self._pipe = None
        self.load()

    # -------- utils: détection des clés --------
    @staticmethod
    def _gen_keys() -> set:
        return set(GenerationConfig().to_dict().keys())

    @staticmethod
    def _pipe_keys(task: str) -> set:
        sig = inspect.signature(pipeline)
        common = {
            "device",
            "device_map",
            "framework",
            "batch_size",
            "torch_dtype",
            "return_full_text",
        }
        return {p.name for p in sig.parameters.values()} | common

    def _split_kwargs(
        self, kwargs: Dict[str, Any]
    ) -> Tuple[Dict[str, Any], Dict[str, Any], Dict[str, Any], Dict[str, Any]]:
        gen_keys = self._gen_keys()
        pipe_keys = self._pipe_keys(self.task)

        pipeline_kwargs: Dict[str, Any] = {}
        generation_kwargs: Dict[str, Any] = {}
        model_kwargs: Dict[str, Any] = {}
        tokenizer_kwargs: Dict[str, Any] = {}

        for k, v in kwargs.items():
            if k.startswith("model_"):
                model_kwargs[k[len("model_") :]] = v
            elif k.startswith("tokenizer_"):
                tokenizer_kwargs[k[len("tokenizer_") :]] = v
            elif k.startswith("pipeline_"):
                pipeline_kwargs[k[len("pipeline_") :]] = v
            elif k in gen_keys:
                generation_kwargs[k] = v
            elif k in pipe_keys:
                pipeline_kwargs[k] = v
            else:
                # Par défaut: on considère que c'est un paramètre de génération
                generation_kwargs[k] = v

        return pipeline_kwargs, generation_kwargs, model_kwargs, tokenizer_kwargs

    # -------- chargement --------
    def load(self) -> None:
        if self._pipe is not None:
            return

        print(f"[Model] Loading model '{self.name}' for task '{self.task}'...")

        # Tokenizer
        if "use_fast" not in self.tokenizer_kwargs:
            self.tokenizer_kwargs["use_fast"] = True
        self._tokenizer = AutoTokenizer.from_pretrained(
            self.name, **self.tokenizer_kwargs
        )

        # pad_token pour certains modèles (OPT…) si absent
        if (
            self._tokenizer.pad_token_id is None
            and self._tokenizer.eos_token_id is not None
        ):
            self._tokenizer.pad_token = self._tokenizer.eos_token

        # Model head selon la task
        model_cls = (
            AutoModelForSeq2SeqLM
            if self.task in {"text2text-generation", "summarization", "translation"}
            else AutoModelForCausalLM
        )

        # Valeurs par défaut "safe"
        self.model_kwargs.setdefault("low_cpu_mem_usage", True)
        self.model_kwargs.setdefault("use_safetensors", True)

        # Chargement modèle (supporte quantization_config, device_map, torch_dtype, etc.)
        self._model = model_cls.from_pretrained(self.name, **self.model_kwargs)

        # Pipeline
        if (
            "device" not in self.pipeline_kwargs
            and "device_map" not in self.pipeline_kwargs
        ):
            self.pipeline_kwargs["device"] = -1  # CPU par défaut
        self._pipe = pipeline(
            self.task,
            model=self._model,
            tokenizer=self._tokenizer,
            **self.pipeline_kwargs,
        )

        print("[Model] Load completed.")

    # -------- inférence --------
    def _apply_to_texts(self, texts: List[str], prompt: str) -> List[str]:
        if self._pipe is None:
            self.load()

        infer_kwargs = dict(self.generation_kwargs)
        # Pour 'text-generation', renvoyer seulement la génération (pas l'input)
        if (
            self.task == "text-generation"
            and "return_full_text" not in infer_kwargs
            and "return_full_text" not in self.pipeline_kwargs
        ):
            infer_kwargs["return_full_text"] = False

        outs: List[str] = []
        for txt in texts:
            inp = f"{prompt}\n\n{txt}".strip()
            result = self._pipe(inp, **infer_kwargs)

            if self.task == "text-generation":
                outs.append(result[0].get("generated_text", ""))
            else:
                if isinstance(result, list):
                    val = (
                        result[0].get("generated_text")
                        or result[0].get("summary_text")
                        or result[0].get("answer")
                        or result[0].get("sequence")
                        or str(result[0])
                    )
                else:
                    val = str(result)
                outs.append(val)

        return outs

    def run(self, dataset, prompt: str, output_col: str | None = None):
        """DataFrame-only: ajoute une colonne avec la réponse du modèle."""
        if not hasattr(dataset, "data"):
            raise ValueError("Le dataset fourni n'a pas d'attribut 'data'.")
        if not isinstance(dataset.data, pd.DataFrame):
            raise TypeError("Le dataset.data doit être un pandas.DataFrame.")
        if not hasattr(dataset, "field"):
            raise ValueError("Le dataset DataFrame doit définir l'attribut 'field'.")
        if dataset.field not in dataset.data.columns:
            raise KeyError(
                f"Colonne '{dataset.field}' introuvable. Colonnes: {list(dataset.data.columns)}"
            )

        df = dataset.data.copy()
        outs = self._apply_to_texts(df[dataset.field].astype(str).tolist(), prompt)

        safe_name = self.name.replace("/", "_").replace("-", "_").replace(".", "_")
        if output_col is None:
            output_col = f"{dataset.field}__{safe_name}"

        base, i = output_col, 1
        while output_col in df.columns:
            output_col = f"{base}_{i}"
            i += 1

        df[output_col] = outs

        result_ds = copy.copy(dataset)
        result_ds.data = df
        result_ds.last_output_col = output_col
        if hasattr(result_ds, "limit"):
            result_ds.limit = len(df)
        if hasattr(result_ds, "name"):
            result_ds.name = f"{getattr(dataset, 'name', 'dataset')}__{safe_name}"
        return result_ds



from datasets import load_dataset
import pandas as pd
from itertools import islice


class Dataset:
    def __init__(self, name: str, split: str, field: str, limit: int):
        self.name = name
        self.split = split
        self.field = field
        self.limit = limit
        self.data = None  # DataFrame (index, texte)

        self.download()

    def download(self):
        print(
            f"[Dataset] Streaming {self.limit} rows from '{self.name}' ({self.split})..."
        )

        # streaming=True => lit ligne par ligne à distance
        ds_iter = load_dataset(self.name, split=self.split, streaming=True)

        rows = []
        for i, row in islice(enumerate(ds_iter), self.limit):
            if self.field not in row:
                raise KeyError(
                    f"Le champ '{self.field}' est introuvable. "
                    f"Champs disponibles: {list(row.keys())}"
                )
            rows.append({"index": i, self.field: row[self.field]})

        self.data = pd.DataFrame(rows)

        print(f"[Dataset] Download completed. Loaded {len(self.data)} rows.")



# tests/test_model.py
import pandas as pd
import pytest

# Adapte l'import si ton fichier s'appelle autrement:
from cleanote.model import Model


# ---------- Doubles de test (mocks légers) ----------
class FakeTokenizer:
    def __init__(self, pad_token_id=None, eos_token_id=1, **kwargs):
        self.pad_token_id = pad_token_id
        self.eos_token_id = eos_token_id
        self.eos_token = "<eos>"
        self.pad_token = None
        self.kwargs = kwargs


class FakeCausalModel:
    def __init__(self, name, **kwargs):
        self.name = name
        self.kwargs = kwargs


class FakeSeq2SeqModel(FakeCausalModel):
    pass


class PipelineRecorder:
    """Capture les kwargs passés à pipeline() et la dernière inférence."""

    def __init__(self, task, model, tokenizer, **kwargs):
        self.task = task
        self.model = model
        self.tokenizer = tokenizer
        self.kwargs = kwargs
        self.calls = []  # liste des inputs reçus

    def __call__(self, inputs, **infer_kwargs):
        # inputs peut être une str (unitaire) dans notre code
        self.calls.append({"inputs": inputs, "infer_kwargs": infer_kwargs})
        if self.task == "text-generation":
            return [{"generated_text": "GEN_OUT"}]
        else:
            # pour text2text-generation, on renvoie un champ différent
            return [{"summary_text": "SEQ2SEQ_OUT"}]


# ---------- Fixtures de monkeypatch ----------
@pytest.fixture
def patch_transformers(monkeypatch):
    created = {}

    def fake_auto_tokenizer_from_pretrained(name, **kwargs):
        created["tokenizer_called_with"] = {"name": name, **kwargs}
        # Simule l'absence de pad_token_id pour tester la logique de fallback
        return FakeTokenizer(pad_token_id=None, eos_token_id=1, **kwargs)

    def fake_causal_from_pretrained(name, **kwargs):
        created["causal_model_called_with"] = {"name": name, **kwargs}
        return FakeCausalModel(name, **kwargs)

    def fake_seq2seq_from_pretrained(name, **kwargs):
        created["seq2seq_model_called_with"] = {"name": name, **kwargs}
        return FakeSeq2SeqModel(name, **kwargs)

    def fake_pipeline(task, model, tokenizer, **kwargs):
        created["pipeline_called_with"] = {"task": task, "kwargs": kwargs}
        return PipelineRecorder(task, model, tokenizer, **kwargs)

    monkeypatch.setattr(
        "cleanote.model.AutoTokenizer.from_pretrained",
        fake_auto_tokenizer_from_pretrained,
    )
    monkeypatch.setattr(
        "cleanote.model.AutoModelForCausalLM.from_pretrained",
        fake_causal_from_pretrained,
    )
    monkeypatch.setattr(
        "cleanote.model.AutoModelForSeq2SeqLM.from_pretrained",
        fake_seq2seq_from_pretrained,
    )
    monkeypatch.setattr("cleanote.model.pipeline", fake_pipeline)

    return created


# ---------- Aides ----------
class FakeDataset:
    def __init__(self, df: pd.DataFrame, field: str = "full_note"):
        self.data = df
        self.field = field
        self.name = "dummy/ds"
        self.limit = len(df)


# ---------- Tests ----------
def test_text_generation_happy_path_df_adds_column_and_sets_pad(
    monkeypatch, patch_transformers, capsys
):
    df = pd.DataFrame({"index": [0, 1], "full_note": ["hello", "world"]})
    ds = FakeDataset(df)

    # both max_new_tokens and max_length -> le second doit être retiré
    m = Model(
        name="facebook/opt-350m",
        task="text-generation",
        max_new_tokens=8,
        max_length=999,  # doit être ignoré
        tokenizer_use_fast=True,
        pipeline_batch_size=4,  # route vers pipeline_kwargs
    )

    # prints « Loading... » + « Load completed. »
    out_txt = capsys.readouterr().out
    assert "Loading model 'facebook/opt-350m' for task 'text-generation'..." in out_txt
    assert "Load completed." in out_txt

    # pad_token doit être positionné par la classe (pad_token_id None, eos_token_id=1)
    # Vérifie que tokenizer_use_fast a bien été pris en compte
    assert patch_transformers["tokenizer_called_with"]["use_fast"] is True

    # run => ajoute une colonne auto-nommée
    out_ds = m.run(ds, prompt="count words please")
    new_cols = [c for c in out_ds.data.columns if c.startswith(ds.field + "__")]
    assert len(new_cols) == 1
    col = new_cols[0]
    assert list(out_ds.data[col]) == ["GEN_OUT", "GEN_OUT"]
    # l'original n'est pas écrasé
    assert list(out_ds.data[ds.field]) == ["hello", "world"]
    # garde un pointeur vers la dernière colonne
    assert out_ds.last_output_col == col

    # pipeline kwargs: device par défaut = -1 si non fourni
    assert patch_transformers["pipeline_called_with"]["kwargs"].get("device", -2) == -1

    # generation_kwargs: max_length doit avoir été retiré si max_new_tokens présent
    assert "max_length" not in m.generation_kwargs
    assert m.generation_kwargs["max_new_tokens"] == 8


def test_text2text_generation_branch(monkeypatch, patch_transformers):
    df = pd.DataFrame({"index": [0, 1], "full_note": ["a", "b"]})
    ds = FakeDataset(df)

    m = Model(name="google/flan-t5-small", task="text2text-generation", num_beams=2)
    out_ds = m.run(ds, prompt="rewrite politely")
    # colonne ajoutée avec la sortie seq2seq
    new_col = [c for c in out_ds.data.columns if c.startswith(ds.field + "__")][0]
    assert list(out_ds.data[new_col]) == ["SEQ2SEQ_OUT", "SEQ2SEQ_OUT"]


def test_output_col_collision_is_suffixed(monkeypatch, patch_transformers):
    df = pd.DataFrame(
        {"index": [0], "full_note": ["x"], "full_note__facebook_opt_350m": ["exists"]}
    )
    ds = FakeDataset(df)

    m = Model(name="facebook/opt-350m", task="text-generation")
    out_ds = m.run(ds, prompt="p")
    # collision => suffix _1
    cols = [
        c for c in out_ds.data.columns if c.startswith("full_note__facebook_opt_350m")
    ]
    assert "full_note__facebook_opt_350m" in cols
    assert "full_note__facebook_opt_350m_1" in cols


def test_run_respects_explicit_output_col(monkeypatch, patch_transformers):
    df = pd.DataFrame({"index": [0, 1], "text": ["x", "y"]})
    ds = FakeDataset(df, field="text")

    m = Model(name="repo/model", task="text-generation")
    out_ds = m.run(ds, "p", output_col="text_out")
    assert "text_out" in out_ds.data.columns
    assert list(out_ds.data["text_out"]) == ["GEN_OUT", "GEN_OUT"]


def test_errors_dataset_without_dataframe(monkeypatch, patch_transformers):
    class BadDs:
        def __init__(self):
            self.data = {"index": [0], "full_note": ["x"]}
            self.field = "full_note"

    m = Model(name="x/y", task="text-generation")
    with pytest.raises(TypeError):
        m.run(BadDs(), "p")


def test_errors_missing_field_attribute(monkeypatch, patch_transformers):
    class BadDs:
        def __init__(self, df):
            self.data = df

    df = pd.DataFrame({"index": [0], "full_note": ["x"]})
    m = Model(name="x/y", task="text-generation")
    with pytest.raises(ValueError):
        m.run(BadDs(df), "p")


def test_errors_missing_column(monkeypatch, patch_transformers):
    df = pd.DataFrame({"index": [0], "other": ["x"]})

    class Ds:
        def __init__(self, df):
            self.data = df
            self.field = "full_note"

    ds = Ds(df)

    m = Model(name="x/y", task="text-generation")
    with pytest.raises(KeyError):
        m.run(ds, "p")


def test_pipeline_return_full_text_not_overridden(monkeypatch, patch_transformers):
    # Si l'utilisateur a déjà mis pipeline_return_full_text=True,
    # _apply_to_texts ne doit PAS forcer return_full_text=False
    df = pd.DataFrame({"index": [0, 1], "full_note": ["u", "v"]})
    ds = FakeDataset(df)

    m = Model(
        name="facebook/opt-350m",
        task="text-generation",
        pipeline_return_full_text=True,  # impose le comportement pipeline
    )

    _ = m.run(ds, "p")
    # récupérer le dernier appel pipeline (enregistré dans fixture)
    # et vérifier que return_full_text n'a PAS été injecté par _apply_to_texts
    # (on ne peut pas accéder direct ici; on valide en re-déclarant un nouveau prompt)
    # -> On force un 2e run et on teste qu'aucun param "return_full_text" n'est ajouté
    out = m._pipe.calls[-1]
    assert "return_full_text" not in out["infer_kwargs"]



# tests/test_pipeline.py
import pandas as pd

# Adapte ces imports selon ton arborescence de projet :
# from ton_module.pipeline import Pipeline
# Ici on suppose pipeline.py à la racine:
from cleanote.pipeline import Pipeline


class FakeDataset:
    """Dataset minimal compatible (DataFrame-only) pour tester Pipeline."""

    def __init__(self, field="full_note"):
        self.field = field
        self.name = "dummy/ds"
        self.limit = 2
        self.data = pd.DataFrame(
            {"index": [0, 1], field: ["hello world", "second row"]}
        )


class FakeModel:
    """Mock de Model avec .run(dataset, prompt, output_col)."""

    def __init__(self):
        self.calls = []  # trace des appels

    def run(self, dataset, prompt, output_col=None):
        # On trace les arguments pour les assertions
        self.calls.append({"prompt": prompt, "output_col": output_col})
        # Retourne une *copie légère* du dataset avec la nouvelle colonne
        out = type(dataset).__new__(type(dataset))
        out.__dict__ = dict(dataset.__dict__)
        df = dataset.data.copy()
        df[output_col] = ["OK-0", "OK-1"]
        out.data = df
        return out


def test_pipeline_apply_happy_path(capsys):
    ds = FakeDataset(field="full_note")
    m_h = FakeModel()
    m_v = FakeModel()  # inutilisé dans la version actuelle du pipeline

    pipe = Pipeline(dataset=ds, model_h=m_h, model_v=m_v)
    out = pipe.apply()

    # Vérifie les prints
    printed = capsys.readouterr().out
    assert "[Pipeline] Starting pipeline..." in printed
    assert "[Pipeline] Start Homogenization..." in printed
    assert "[Pipeline] Homogenization completed." in printed
    assert "[Pipeline] Pipeline completed." in printed

    # Vérifie que model_h.run a été appelé avec le bon prompt + output_col
    assert len(m_h.calls) == 1
    call = m_h.calls[0]
    assert call["prompt"].startswith("please give me the number of words")
    assert call["output_col"] == f"{ds.field}__h"

    # Vérifie le DataFrame de sortie: colonne ajoutée, contenu cohérent
    assert out is not ds  # retourne un *nouveau* dataset
    assert out.field == ds.field  # le champ est préservé
    new_col = f"{ds.field}__h"
    assert new_col in out.data.columns
    assert list(out.data[new_col]) == ["OK-0", "OK-1"]

    # L'original n'est pas modifié
    assert new_col not in ds.data.columns


def test_pipeline_no_side_effect_on_input():
    ds = FakeDataset()
    m_h = FakeModel()
    pipe = Pipeline(ds, m_h, FakeModel())
    _ = pipe.apply()

    # Le dataset d'entrée ne doit pas être modifié
    assert list(ds.data.columns) == ["index", ds.field]



import pandas as pd
import pytest

from cleanote.dataset import Dataset


def make_stream_iter(rows):
    def _iter():
        for r in rows:
            yield r

    return _iter()


def test_download_happy_path(monkeypatch):
    captured = {}

    def fake_load_dataset(name, split=None, streaming=False, **kwargs):
        captured["name"] = name
        captured["split"] = split
        captured["streaming"] = streaming
        rows = [{"text": f"note {i}", "other": i} for i in range(5)]
        return make_stream_iter(rows)

    monkeypatch.setattr("cleanote.dataset.load_dataset", fake_load_dataset)

    ds = Dataset(name="dummy/repo", split="train", field="text", limit=3)

    assert captured["name"] == "dummy/repo"
    assert captured["split"] == "train"
    assert captured["streaming"] is True

    assert isinstance(ds.data, pd.DataFrame)
    assert len(ds.data) == 3
    assert list(ds.data.columns) == ["index", "text"]
    assert ds.data.loc[0, "index"] == 0
    assert ds.data.loc[0, "text"] == "note 0"
    assert ds.data.loc[2, "text"] == "note 2"


def test_download_missing_field_raises(monkeypatch):
    def fake_load_dataset(name, split=None, streaming=False, **kwargs):
        rows = [{"text": "ok"}, {"text": "ok 2"}]
        return make_stream_iter(rows)

    monkeypatch.setattr("cleanote.dataset.load_dataset", fake_load_dataset)

    ds = Dataset.__new__(Dataset)
    ds.name = "dummy/repo"
    ds.split = "train"
    ds.field = "missing"
    ds.limit = 2
    ds.data = None

    with pytest.raises(KeyError) as exc:
        ds.download()
    assert "introuvable" in str(exc.value)
    assert ds.data is None


def test_download_zero_limit(monkeypatch):
    def fake_load_dataset(name, split=None, streaming=False, **kwargs):
        rows = [{"text": "a"}, {"text": "b"}]
        return make_stream_iter(rows)

    monkeypatch.setattr("cleanote.dataset.load_dataset", fake_load_dataset)

    ds = Dataset.__new__(Dataset)
    ds.name = "dummy/repo"
    ds.split = "train"
    ds.field = "text"
    ds.limit = 0
    ds.data = None

    ds.download()

    assert isinstance(ds.data, pd.DataFrame)
    assert len(ds.data) == 0
    # DataFrame vide ⇒ pas de colonnes
    assert list(ds.data.columns) == []
